// GenoSet.cpp: implementation of the GenoSet class.
//
//////////////////////////////////////////////////////////////////////
#include "HaploSet.h"
#include "GenoSet.h"
#include <iomanip>
#include <stack>

extern int gLength, gSize, gOriginLen;
extern bool bCutSNP;
extern HaploSet hapset, preHapset;
extern GenoSet gset, originGset;
extern haplotype * xHap;
extern ofstream outf, resultf;
extern int * SNPmap;
extern vector<haplotype *> solution;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GenoSet::GenoSet()
{

}

GenoSet::~GenoSet()
{
	m_gvec.clear ();
}

void GenoSet::readFile(ifstream &genofile)
{	
	char line[500];
	int num;
	genotype * g;
	genofile >> num >> gLength;
	gSize = (gLength-1) / 8 +1;
	
	for(int i=0; i< num; i++)	{
		if(!(genofile >> line))	{
			genofile.close();
			cout << "Bad format of input genotypes file!" << endl;
			exit(-1);
		}
		g = new genotype();
		strncpy(g->m_sequence, line, gLength);
		g->m_index = i+1;
		m_gvec.push_back (g);
	}

	return;
}

//pre-process procedure, remove (and record) identical SNP columns
//called by OriginGset
void GenoSet::cutSNP()
{
	gOriginLen = gLength;
	int i, j, pre, next, size = m_gvec.size();
	char ** SNP = new char*[gOriginLen];	
	bool * isDup = new bool[gOriginLen];
	SNPmap = new int[gOriginLen];
	
	for(i=0; i<gOriginLen; i++)	{
		SNPmap[i] = -1;
		isDup[i] = false;
		SNP[i] = new char[size+1];
		memset(SNP[i], 0, size+1);
	}

	for(j=0; j<size; j++)	{
		for(i=0; i<gOriginLen; i++)	
			SNP[i][j] = m_gvec[j]->m_sequence[i];
	}

	//set SNPmap: id SNP list, points to next id SNP, and -1 when ends
	for(i=0; i<gOriginLen-1; i++)	{
		if(!isDup[i])	{
			pre =i;
			for(j=i+1; j<gOriginLen; j++)	{
				if((!isDup[j])&&(!strcmp(SNP[i], SNP[j])))	{
					SNPmap[pre] = j;					
					pre = j;
					isDup[j] = true;
				}
				//SNPmap[pre] = -1;
			}
		}
	}

	gLength=0;
	for(i=0; i<gOriginLen; i++)	{
		if(!isDup[i])	{			
			for(j=0; j<size; j++)	
				m_gvec[j]->m_sequence[gLength] = m_gvec[j]->m_sequence[i];
			pre =i;
			while(SNPmap[pre]!=-1)	{
				next = SNPmap[pre];
				SNPmap[pre] = gLength;
				pre = next;
			}
			SNPmap[pre] = gLength;
			gLength++;
		}
	}

	for(j=0; j<size; j++)
		memset(m_gvec[j]->m_sequence+gLength, 0, gOriginLen-gLength);
	
	gSize = (gLength-1) / 8 +1;	

	for(i=0; i<gOriginLen; i++)		delete SNP[i];	
	delete SNP;
	delete isDup;
	return;
}

void GenoSet::print(ostream &os, bool bCut)
{
	os << "Genotypes " << m_gvec.size();
	if(bCut)	os << " (compact)" << endl;
	else		os << " (full length)" << endl;
	genoit it;
	for(it=m_gvec.begin(); it!=m_gvec.end(); it++)
		if(bCut)	os << **it << endl;
		else	{
			(*it)->printSeq(os,false);
			os << endl;
		}
	return;
}

//finalizing, call by OriginGenoSet to delete all genotypes
void GenoSet::delGenotypes()
{
	genoit it;
	for(it=m_gvec.begin(); it!=m_gvec.end(); it++)
		delete *it;
	return;
}

//called only by originGset
//identify preChosen haplotypes from homozygotes or single-heterozygotes
//use these haplotypes to further reslove other genotypes
//and remove all these resolved genotypes 
void GenoSet::preResolve()
{
	genoit it;
	int i;

	//identify preChosen haplotypes from homozygotes or single-heterozygotes
	for(it=m_gvec.begin(); it!=m_gvec.end(); it++)	{
		int num2=(*it)->get2siteNum();
		if(num2<=1)	{
			haplotype h1, h2;
			for(i=0; i<gLength; i++)	{
				if((*it)->m_sequence[i]=='0')		{
					h1.setPosition (i, 0);
					h2.setPosition (i, 0);
				}
				else if((*it)->m_sequence[i]=='1')		{
					h1.setPosition (i, 1);
					h2.setPosition (i, 1);
				}
				else	{
					h1.setPosition (i, 0);
					h2.setPosition (i, 1);
				}
			}
			if(num2==0)		{
				h1.m_bChosen = true;
				//h1.m_coverage+=2;
				//h1.m_homo = true;
				const haplotype * ph1= preHapset.insert (h1);			
				(*it)->m_resolutions.push_front (make_pair(ph1, ph1));				
			}
			else	{
				h1.m_bChosen = true;										
				//h1.m_coverage++;
				const haplotype * ph1= preHapset.insert (h1);						
				h2.m_bChosen = true;
				//h2.m_coverage++;
				const haplotype * ph2 = preHapset.insert (h2);
				(*it)->m_resolutions.push_back (make_pair(ph1, ph2));
			}
			(*it)->m_resolveState = 1;
			//cout << "g" << (*it)->m_index << "resolved: " << *ph1 << " + " << *ph2 << endl;			
		}
	}
	//hapset.print(cout);

	//use resolved haplotypes to resolve other genotypes
	hapit hit;	
	for(it=m_gvec.begin (); it!=m_gvec.end(); it++)	{
	if((*it)->m_resolveState ==0)	{
		for(hit=preHapset.m_hset.begin(); hit!=preHapset.m_hset.end(); hit++)	{
			haplotype h;
			if((*it)->resolve(*hit, h))	{				
				const haplotype * ph= preHapset.find (h);
				if(ph!=0)	{
					(*it)->m_resolutions.push_back (make_pair(&(*hit), ph));				
					(*it)->m_resolveState = 1;
				}
			}
		}		
	}
	}	
	//print(cout);

	//add remaining unresolved genotypes to gset
	for(it=m_gvec.begin (); it!=m_gvec.end(); it++)	{
		if((*it)->m_resolveState ==0)	
			//cout << "unresolved: g" << (*it)->m_index << endl;
			gset.m_gvec.push_back(*it);		
	}
	gset.sortBy2();
//	outf << "after preprocessing (resolve homo or 1-heterogoze, sorting)" << endl;
//	gset.print(outf);
//	preHapset.print(outf);

	return;
}

void GenoSet::buildGHgraph()
{
	char * g1, * g2;
	haplotype h0, h1, pairh;
	vector<int> sites;
	int k, pos, i,j;
	bool first;

	//process every gi&gj
	for(i=0; i< m_gvec.size()-1; i++)		{
		g1= m_gvec[i]->m_sequence;
		//if gi is contained in gj, then process gi, instead of gi&gj for all j
		for(j=m_gvec.size()-1; j>i; j--)		{
			if(m_gvec[i]->isContained(*(m_gvec[j])))	{
				sites.clear();
				for(pos=0; pos<gLength; pos++)	{
					if(g1[pos]=='2')	{
						sites.push_back(pos);				
						h0.setPosition(pos, 0);
					}
					else 
						h0.setPosition(pos, g1[pos]-'0');					
				}
				first =true;
				while(true)	{
					m_gvec[i]->resolve(h0,h1);					
					if(first)	{
						setHrelation(h0, true);
						setHrelation(h1, true);												
					}
					else
						setH2relation(h0, h1, m_gvec[i], m_gvec[j]);
					k=0;
					while(h0.getPosition(sites[k])==1)	{
						h0.setPosition(sites[k++], 0);
						if(k==sites.size()-1)	break;
					}
					if(k==sites.size()-1)		break;
					h0.setPosition(sites[k], 1);
					first = false;
				}				
				break;
			}
		}
		if(j>i)	continue;

		//process every gi&gj for all j>i
		for(j=i+1; j< m_gvec.size(); j++)		{
			g2= m_gvec[j]->m_sequence;
			sites.clear ();
			for(pos=0; pos<gLength; pos++)	{
				if((g1[pos]=='2')&&(g2[pos]=='2'))	{
					sites.push_back(pos);				
					h0.setPosition(pos, 0);
				}
				else if(g1[pos]=='2')	
					h0.setPosition(pos, g2[pos]-'0');
				else if(g2[pos]=='2')	
					h0.setPosition(pos, g1[pos]-'0');
				else if(g2[pos]==g1[pos])				
					h0.setPosition(pos, g1[pos]-'0');
				else 
					break;				
			}
			if(pos< gLength)	//g1&g2 not compatible
				continue;
			//generate exp(2,sites.size()) haplotypes
			if(sites.size()==0)	{
				setHrelation(h0);
				continue;
			}
			first = true;
			while(true)	{				
				setHrelation(h0, first);
				k=0;
				while(h0.getPosition(sites[k])==1)	{
					h0.setPosition(sites[k++], 0);
					if(k==sites.size())	break;
				}
				if(k==sites.size())		break;
				h0.setPosition(sites[k], 1);
				first = false;
			}
		}
	}
	outf << "Candidate haplotypes & genotype resolution" << endl;
	hapset.print(outf);
	print(outf);
	return;
}

//set h.m_relation according to gset, also update resolution of genotypes
void GenoSet::setHrelation(haplotype &h, bool first)
{
	genoit it, it1;
	haplotype pairh;
	bool bAmbuguous=true;
	list<pair<genotype*, const haplotype*> > ghlist;
	list<pair<genotype*, const haplotype*> >::iterator ghit; 
	const haplotype * ph;//pointer to h'=pairh in hapset

	//if h has been added to hapset, return
	if(hapset.find(h)!=0)	return;

	h.m_relation.clear();
	h.m_priority =0; 
	//outf << endl << h << endl;
	for(it=m_gvec.begin (); it!=m_gvec.end(); it++)	{					
		//if h can resolve genotype *it
		if((*it)->resolve(h, pairh))	{				
			//outf << "  g" << (*it)->m_index << " : " << pairh;
			h.addRelation(*it);
			ph= hapset.find (pairh);
			//if counterpart h' (pairh) has been in hapset, add g to h, record (g,h')
			if(ph!=0)	{				
				ghlist.push_back(make_pair(*it, ph));//(*it)->m_resolutions.push_back (make_pair(&h, ph));				
				bAmbuguous = false;				
				if(ph->m_priority==9999)	h.m_priority += 1; 
				else if(ph->m_priority==10000)	h.m_priority += 2; 
			}
			//if h' has not been in hapset, check coverage of h'
			else	{				
				if(cover1(pairh, *it))
					ghlist.push_back(make_pair(*it, xHap));
				else
					bAmbuguous = false;
			}
		}		
	}

	//still have chance to be added
	if(h.m_relation.size()>2)	{
		bAmbuguous = false;
	}

	//add h into hapset and fill m_resolution of g in ghlist
	if((!bAmbuguous)||first)	{
		h.m_coverage = h.m_relation.size();
		const haplotype * newh = hapset.insert(h);
		for(ghit=ghlist.begin(); ghit!=ghlist.end(); ghit++)	{
			//if(((*ghit).second!=xHap)&&((*ghit).second->m_homo))
			//	(*ghit).first->m_resolutions.push_front(make_pair(newh, (*ghit).second));
			//else			
			//	(*ghit).first->m_resolutions.push_back(make_pair(newh, (*ghit).second));
			(*ghit).first->addResolution(newh, (*ghit).second);
		}
	}
	ghlist.clear();

	return;
}

//set h1/h2.m_relation according to gset, also update resolution of genotypes
void GenoSet::setH2relation(haplotype &h1, haplotype & h2, genotype * g1, genotype * g2)	{
	haplotype pairh;
	if(cover2(h1,g1,g2)&&cover2(h2,g1,g2))	{
		g2->resolve(h1, pairh);
		if(cover1(pairh, g2))	{
			g2->resolve(h2, pairh);
			if(cover1(pairh, g2))	{
				return;
			}		
		}		
	}	
	setHrelation(h1);
	setHrelation(h2);		
	return;
}


//true if haplotype h's coverage =1
bool GenoSet::cover1(haplotype &h, genotype * g)
{
	genoit it;
	for(it=m_gvec.begin(); it!=m_gvec.end(); it++)	{
		if(((*it)!=g)&&((*it)->resolve(h)))		return false;
	}
	return true;
}

//true if haplotype h's coverage =2
bool GenoSet::cover2(haplotype &h, genotype * g1, genotype * g2)
{
	genoit it;
	for(it=m_gvec.begin(); it!=m_gvec.end(); it++)	{
		if(((*it)!=g1)&&((*it)!=g2)&&((*it)->resolve(h)))		return false;
	}
	return true;
}

//complete m_relation of preChosen haplotypes, also that of g if h' coverage=1
void GenoSet::setPreRelation()	{	
	hapit hit;
	genoit it;
	haplotype h, pairh;	
	list<pair<genotype*, const haplotype*> > ghlist;
	list<pair<genotype*, const haplotype*> >::iterator ghit; 
	list<resolution>::iterator rit;

	for(hit=preHapset.m_hset.begin(); hit!=preHapset.m_hset.end(); hit++)	{
		h = *hit;
		//compute coverage of pre-resolved genotypes
		h.m_priority =9999;
		for(it=originGset.m_gvec.begin(); it!=originGset.m_gvec.end(); it++)	{
			//cout << **it << endl;
			for(rit=(*it)->m_resolutions.begin(); rit!=(*it)->m_resolutions.end(); rit++)	{				
				if(((&(*hit))==(*rit).first)&&((&(*hit))==(*rit).second))	{
					h.m_coverage +=2;
					h.m_priority =10000;
				}
				else if((&(*hit))==(*rit).first)	
					h.m_coverage++;
				else if((&(*hit))==(*rit).second)	
					h.m_coverage++;
			}
		}
		//compute coverage of remained genotypes
		for(it=m_gvec.begin(); it!=m_gvec.end(); it++)	{			
			if((*it)->resolve(h, pairh))	{				
				h.addRelation(*it);
				if(cover1(pairh,*it))
					ghlist.push_back(make_pair(*it, xHap));
			}
		}
		h.m_coverage += h.m_relation.size();
		const haplotype * newh = hapset.insert(h);
		for(ghit=ghlist.begin(); ghit!=ghlist.end(); ghit++)	{
			//if(newh->m_homo)
			//	(*ghit).first->m_resolutions.push_front(make_pair(newh, (*ghit).second));
			//else
			//	(*ghit).first->m_resolutions.push_back(make_pair(newh, (*ghit).second));
			(*ghit).first->addResolution(newh, (*ghit).second);
		}
		ghlist.clear();
	}		
	outf << "pre-Chosen haplotypes and their resolution" << endl;
	hapset.print(outf);
	print(outf);
	return;
}

//sort genotypes in m_gvec according to #SNP=2 (increasing order)
void GenoSet::sortBy2()
{
	vector<genotype*> vec;
	vec = m_gvec;
	m_gvec.clear ();
	int i, j, minkey, mink, size=vec.size();
	int * key=new int[size];
	bool * sorted = new bool[size];
	
	for(i=0; i< size; i++)		{
		key[i]= vec[i]->get2siteNum();	
		sorted[i] = false;
	}

	while(size>0)	{
		minkey =gLength+1;
		for(i=0; i<vec.size(); i++)	
			if((!sorted[i])&&(key[i]<minkey))	
			{	minkey =key[i];	mink =i;	}
		m_gvec.push_back(vec[mink]);
		sorted[mink]= true;
		size--;
	}

	delete key;
	delete sorted;

	return;
}


//given GHgraph, resolve genotypes by greedy strategy. 
//return hap number (preChosen haplotypes excluded, unresolved genotypes excluded)
//(for each genotype, choose hap-pair with max total coverage
int GenoSet::greedyResolve()
{
	set<const haplotype *> gsolution;
	set<const haplotype *>::iterator it;
	list<resolution>::iterator rit;
	int maxc, c, unresolved;
	const haplotype * ph1, *ph2;

	outf << "Solution by Greedy:" << endl;

	for(int i=0; i<m_gvec.size(); i++)	{		
		//cout << *(m_gvec[i]);
		maxc =0;
		for(rit=m_gvec[i]->m_resolutions.begin();rit!=m_gvec[i]->m_resolutions.end();rit++)	{
			//cout << *((*rit).first) << " " << (*rit).first->m_coverage << ";  " << *((*rit).second) << " " << (*rit).second->m_coverage;
			if((*rit).second==xHap)	
				c =(*rit).first->m_coverage +1;
			else
				c =(*rit).first->m_coverage + (*rit).second->m_coverage;
			if((*rit).first->m_bChosen||(((*rit).second!=xHap)&&(*rit).second->m_bChosen))
				c+=1;
			if(c>maxc)	{
				maxc = c;
				ph1 = (*rit).first;
				ph2 = (*rit).second;				
			}
		}
		outf << setw(3) << m_gvec[i]->m_index << ": ";
		m_gvec[i]->printSeq(outf, false);
		//no resolution (any resolution is OK)
		if(maxc ==0)	{
			unresolved++;
			outf << " unresolved" << endl;
		}
		else	{
			outf << " = ";
			ph1->printSeq(outf, false);
			outf << "(" << ph1->m_index << ") + " ;
			if(ph2==xHap)	{
				haplotype * tmph = new haplotype();
				m_gvec[i]->resolve(*ph1, *tmph);
				tmph->m_coverage =1;
				ph2 = tmph;				
			}
			ph2->printSeq(outf, false);
			outf << "(" << ph2->m_index << ")" << endl;
			if(!(ph1->m_bChosen))		
				gsolution.insert(ph1);
			if(!(ph2->m_bChosen))		gsolution.insert(ph2);
		}
	}

	copyHset(solution, gsolution);
	outf << "haplotypes " << gsolution.size() << endl;
	for(it=gsolution.begin(); it!=gsolution.end(); it++)	{
		outf << setw(3) << (*it)->m_index << ": "; 
		(*it)->printSeq(outf, false);		
		if((*it)->m_index==-1)	delete (*it);
		outf << endl;
	}
	
	return gsolution.size();
}

//resolve by depth-first searching (branch & bound)
void GenoSet::searchResolve()
{
	//remove unresolved genotypes (no esolution, or any resolution)
	genoit it;
	//print(cout);
	for(it=m_gvec.begin(); it!=m_gvec.end(); it++)	{
		if((*it)->m_resolutions.size()==0)	{
			(*it)->m_resolveState =2;
			m_gvec.erase(it);
			it--;
		}
	}
	//print(cout);

	int hnum = greedyResolve();
	int gnum = hnum;

	typedef pair<list<resolution>::iterator, set<const haplotype *> > stack_node;
	set<const haplotype *> partsol;	
	set<const haplotype *>::iterator phit;
	stack<stack_node> solStack;	
	stack_node node;
	bool bFinish = false;

	int ssize=0, gsize=m_gvec.size();
	list<resolution>::iterator rit;
	//print(cout);
	while(ssize<gsize)	{
		rit=m_gvec[ssize]->m_resolutions.begin();
		if(!((*rit).first ->m_bChosen))		
			partsol.insert((*rit).first);
		if((*rit).second==xHap)	{
			haplotype * tmph = new haplotype();
			m_gvec[ssize]->resolve(*((*rit).first), *tmph);
			tmph->m_coverage =1;
			tmph->m_index = -ssize-1;
			partsol.insert(tmph);
		}
		else if(!((*rit).second->m_bChosen))		
			partsol.insert((*rit).second);
		
		solStack.push(make_pair(rit, partsol));
		ssize++;	
		//cout << "stack: " << ssize << "; top: ";
		//for(phit=partsol.begin(); phit!=partsol.end(); phit++)	
		//	cout << (*phit)->m_index << " ";
		//cout << endl;
		if(partsol.size()>=hnum)	break;
	}
	if((ssize==gsize)&&(partsol.size()<hnum))		{
		hnum = partsol.size();
		copyHset(solution, partsol);
			node.second.clear();
			while(solStack.top().second.size()>=hnum){
				for(phit=node.second.begin(); phit!=node.second.end(); phit++)	
					if((*phit)->m_index==-ssize-1)
						delete (*phit);			
				node = solStack.top();
				solStack.pop();
				ssize--;
				if(ssize==0)	break;
			}			
			solStack.push(node);
			ssize++;
			//cout << "stack: " << ssize << "; top: ";
			//for(phit=partsol.begin(); phit!=partsol.end(); phit++)	
			//	cout << (*phit)->m_index << " ";
			//cout << endl;			
	}

	while(true)	{
		bFinish = true;
		while(true)	{
			if(ssize==0)	break;
			rit=solStack.top().first;			
			for(phit=solStack.top().second.begin(); phit!=solStack.top().second.end(); phit++)	
				if((*phit)->m_index==-ssize)
					delete (*phit);
			solStack.pop();
			ssize--;
			if(++rit!=m_gvec[ssize]->m_resolutions.end())		{
				bFinish = false;
				break;
			}
		}
		if(bFinish)		break;
		
		if(ssize==0)
			partsol.clear();
		else
			partsol = solStack.top().second;
		while(ssize<gsize)	{		
			if(!((*rit).first ->m_bChosen))		
				partsol.insert((*rit).first);
			if((*rit).second==xHap)	{
				haplotype * tmph = new haplotype();
				m_gvec[ssize]->resolve(*((*rit).first), *tmph);
				tmph->m_coverage =1;
				tmph->m_index = -ssize-1;
				partsol.insert(tmph);
			}
			else if(!((*rit).second->m_bChosen))		
				partsol.insert((*rit).second);
			
			solStack.push(make_pair(rit, partsol));
			ssize++;		
			if(ssize<gsize)	rit=m_gvec[ssize]->m_resolutions.begin();
			/*if(ssize<=3)	{
				outf << "stack: " << ssize << "; top: ";
				for(phit=partsol.begin(); phit!=partsol.end(); phit++)	
					outf << (*phit)->m_index << " ";
				outf << endl;			
			}*/
			if(partsol.size()>=hnum)	break;
		}
		if((ssize==gsize)&&(partsol.size()<hnum))		{
			cout << "stack: " << ssize << "; top: ";
			for(phit=partsol.begin(); phit!=partsol.end(); phit++)	
				cout << (*phit)->m_index << " ";
			cout << endl;			
			hnum = partsol.size();
			copyHset(solution, partsol);
			node.second.clear();
			while(solStack.top().second.size()>=hnum){
				for(phit=node.second.begin(); phit!=node.second.end(); phit++)	
					if((*phit)->m_index==-ssize-1)
						delete (*phit);						
				node = solStack.top();
				solStack.pop();
				ssize--;
				if(ssize==0)	break;
			}			
			solStack.push(node);
			ssize++;			
		}
	}

	//if(gnum==hnum)	{
	//	outf << "Exact search get the same result as greedy." << endl;
	//	return;
	//}
	hapit hit;
	for(hit=preHapset.m_hset.begin(); hit!=preHapset.m_hset.end(); hit++)
		solution.push_back(new haplotype(*hit));
	outf << "Solution by Search (branch & bound): " << endl;	
	outf << "haplotypes " << solution.size() << endl;
	resultf << solution.size() << " " << gOriginLen << endl;
	vector<haplotype *>::iterator solit;
	for(solit=solution.begin(); solit!=solution.end(); solit++)	{
		outf << setw(3) << (*solit)->m_index << ": "; 
		(*solit)->printSeq(outf, false);
		(*solit)->printSeq(resultf, false);
		if(!(*solit)->m_bChosen)	{
			outf << "?";
			//resultf << "?";
		}
		//if((*phit)->m_index<0)	delete (*solit);		
		outf << endl;		
		resultf << endl;
	}
	originGset.printSolution(solution);
	outf << endl;	

	return;
}

//given a solution (a set of haplotypes, including preresolved haps)
//output how all genotypes are resolved (called by originGset)
void GenoSet::printSolution(vector<haplotype *> & solution)	{
	vector<haplotype *>::iterator it, it1;
	haplotype pairh;
	typedef list<pair<haplotype*, haplotype*> > phase;
	phase::iterator sit;
	phase * phases = new phase[originGset.m_gvec.size()];
	int i;

	for(i=0; i<m_gvec.size(); i++)	{				
		//if(m_gvec[i]->m_resolveState ==0)	{	
			m_gvec[i]->m_resolutions.clear();
			for(it=solution.begin(); it!=solution.end(); it++)	{
			if(m_gvec[i]->resolve(*(*it), pairh))	
				for(it1=it; it1!=solution.end(); it1++)	{
					if((**it1)==pairh)	{
						(*it)->m_coverage++;
						(*it1)->m_coverage++;
						phases[i].push_back(make_pair(*it, *it1));
						//break;
					}
				}				
			}			
		//}
	}

	resultf << endl << "Resolution of input Genotypes:" << endl;
	resultf << "Genotype" << setw(gLength+10) << "Resolution" << endl;
	for(i=0; i<m_gvec.size(); i++)	{
		//resultf << setw(3) << m_gvec[i]->m_index << ": ";
		m_gvec[i]->printSeq(resultf, false);
		resultf << " :   ";		
		if(m_gvec[i]->m_resolveState ==2)	{			
			resultf << "unresolved" << endl;
		}
		//else if(m_gvec[i]->m_resolveState ==1)	{
		//	outf << "pre-resolved" << endl;
		//}
		else	{		
		for(sit=phases[i].begin(); sit!=phases[i].end(); sit++)	{
			(*sit).first->printSeq(resultf, false);			
			//outf << "(" << (*sit).first->m_index << ") + " ;
			//resultf << "(" << (*sit).first->m_index << ") + " ;
			resultf << " + ";
			(*sit).second->printSeq(resultf, false);			
			//outf << "(" << (*sit).second->m_index << ") " ;						
			//resultf << "(" << (*sit).second->m_index << ") " ;	
			resultf << " ; ";
			if((!(*sit).first->m_bChosen)&&(!(*sit).second->m_bChosen)&&((*sit).first->m_coverage==1)&&((*sit).second->m_coverage==1))	{
				outf << "?  ";
			}
			else	{
				(*sit).first->m_bChosen =(*sit).second->m_bChosen= true;
			}
		}
		phases[i].clear();
		resultf << endl;
		}
	}

	delete[] phases;
	return;
}

//copy s to news, new haplotype and insert to news
//s unchanged, release haplotypes in news before copy
void copyHset(vector<haplotype*>& news, set<const haplotype*>& s)	{
	delHset(news);	
	set<const haplotype *>::iterator it;
	for(it=s.begin(); it!=s.end(); it++)	{
		haplotype * h=new haplotype(**it);
		h->m_coverage =0;
		news.push_back(h);		
	}
	return;
}


//release haplotypes of index<0 in s
void delHset(vector<haplotype*>& s) {
	vector<haplotype *>::iterator it;
	for(it=s.begin(); it!=s.end(); it++)			
			delete *it;	
	s.clear();
	return;
}
