// genotype.cpp: implementation of the genotype class.
//
//////////////////////////////////////////////////////////////////////
#include "haplotype.h"
#include "genotype.h"
#include <iomanip>

extern int gLength, gSize, gOriginLen, *SNPmap;
extern haplotype * xHap;

ostream& operator <<(ostream &os, const genotype &g)	{
	os << setw(3) << g.m_index << " " << g.m_sequence ;
	list<resolution>::const_iterator it = g.m_resolutions.begin ();
	for(; it!=g.m_resolutions.end();it++)	{
		if((*it).second==xHap)	
			os << "  (" << (*it).first->m_index << ", x)";		
		else 
			os << "  (" << (*it).first->m_index << "," <<  (*it).second->m_index << ")";				
	}
	return os;
}


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

genotype::genotype()
{
	m_sequence = new char[gLength+1];
	memset(m_sequence, 0, gLength+1);
	m_index = -1;
	m_resolveState =0;
	//m_bResolved = false;
}

genotype::~genotype()
{
	if(m_sequence!=0)	{
		delete m_sequence;
		m_sequence =0;
	}
	m_resolutions.clear ();
}

int genotype::get2siteNum()
{
	int count=0;
	for(int i=0; i<gLength;i++)	{
		if(m_sequence[i]=='2')	count++;
	}
	return count;
}

//given a haplotype h1, judge if this genotype can be resolve by h1
//if yes, return true, and fill out h2
//note: h2 should be newed before calling, when false, h2 make no sense
bool genotype::resolve(const haplotype &h1, haplotype &h2)
{
	for(int i=0; i<gLength; i++)	{
		int gsite = m_sequence[i]-'0', hsite=h1.getPosition(i);	//convert '0'/'1/'2' to 0/1/2
		if(gsite<2)	{
			if(gsite!=hsite)	return false;
			else	h2.setPosition(i, gsite);
		}
		else //if(site==2)
			h2.setPosition(i, gsite-hsite-1);		
	}
	return true;
}

//similar to resolve(h1, h2), but do not fill h2
bool genotype::resolve(const haplotype &h)
{
	for(int i=0; i<gLength; i++)	{
		int gsite = m_sequence[i]-'0';	//convert '0'/'1/'2' to 0/1/2
		if((gsite<2)&&(gsite!=h.getPosition(i)))	return false;				
	}
	return true;
}

bool genotype::isContained(genotype &g)
{
	int c1, c2;
	for(int i=0; i<gLength; i++)	{
		c1=m_sequence[i]-'0';
		c2= g.m_sequence[i]-'0';
		if(((c1==2)&&(c2<2))||(c1+c2==1))
			return false;
	}
	return true;
}

void genotype::printSeq(ostream &os, bool bCut)
{
	if(bCut||(SNPmap==0))
		os << m_sequence;
	else	{
		for(int i=0; i< gOriginLen; i++)	
			os << m_sequence[SNPmap[i]];
	}
	return;
}


//add a resolution into m_resolution list
void genotype::addResolution(const haplotype *h1, const haplotype *h2)
{
	//swap h1&h2 if necessary	
	if(h1->priority(h2)==-1)	{
		const haplotype * tmp=h2;
		h2 =h1;
		h1=tmp;
	}
	list<resolution>::iterator rit;
	for(rit=m_resolutions.begin(); rit!=m_resolutions.end(); rit++)	{
		if(h1->priority ((*rit).first)==1)
			break;
		if((h2!=xHap)&&((h1->priority ((*rit).first)==0)&&(h2->priority ((*rit).second)==1)))
			break;			
	}
	m_resolutions.insert(rit, make_pair(h1, h2));

	return;
}
