// haplotype.cpp: implementation of the haplotype class.
//
//////////////////////////////////////////////////////////////////////

#include "haplotype.h"
#include "genotype.h"
#include <iomanip>

extern int gSize, gLength, gOriginLen, * SNPmap;
extern haplotype * xHap;

ostream& operator <<(ostream &os, const haplotype &h)	{
	os << setw(3) << h.m_index << " (cover: " << h.m_coverage << "; prior: " << h.m_priority << "): ";
	h.printSeq (os);	
	//list<relation>::const_iterator it = h.m_relation.begin ();
	//for(; it!=h.m_relation.end();it++)	
	//	os << "  (g" << (*it).first->m_index << "," << (*it).second->m_index << ")";		
	list<genotype*>::const_iterator it = h.m_relation.begin ();
	for(; it!=h.m_relation.end();it++)	
		os << "  g" << (*it)->m_index;		
	return os;
}

istream& operator >>(istream &is, haplotype &h)	{
	//is >> ;	
	return is;
}

bool operator < (const haplotype &h1, const haplotype &h2)	{
	for(int i=0; i<gSize; i++)
		if(h1.m_sequence [i] > h2.m_sequence [i])	
			return false;
		else if(h1.m_sequence [i] < h2.m_sequence [i])	
			return true;
	return false;
}

bool operator == (const haplotype &h1, const haplotype &h2)	{
	for(int i=0; i<gSize; i++)
		if(h1.m_sequence [i] != h2.m_sequence [i])	
			return false;		
	return true;
}

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

haplotype::haplotype()
{	
	m_sequence = new unsigned char[gSize+1];	
	memset(m_sequence, 0, gSize+1);
	m_bChosen= false; //m_bComplete= m_bAmbiguous= false;
	m_index = -1;
	//m_homo =false;
	m_priority = 0;
	m_coverage =0;
}

haplotype::haplotype(const haplotype &h)
{
	m_sequence = new unsigned char[gSize+1];	
	for(int i=0; i< gSize+1; i++)
		m_sequence[i] = h.m_sequence [i];
	m_relation = h.m_relation;
	m_bChosen=h.m_bChosen ;
	m_priority = h.m_priority;
	//m_homo = h.m_homo;
	//m_bComplete= h.m_bComplete;
	//m_bAmbiguous = h.m_bAmbiguous;
	m_index = h.m_index;
	m_coverage = h.m_coverage;
}

haplotype::~haplotype()
{
	if(m_sequence!=0)	{
		delete m_sequence;
		m_sequence =0;
	}
	m_relation.clear ();
}

void haplotype::addRelation(genotype * g)
{
	//might need to insert in order
	m_relation.push_back (g);
	return;
}

haplotype& haplotype::operator =(const haplotype &h)
{
	for(int i=0; i< gSize; i++)
		m_sequence[i] = h.m_sequence [i];
	//	const_iterator it=h.m_relation.begin();
	//	for(it=h.m_relation.begin(); it!=h.m_relation.end(); it++)	{
	//		m_relation.push_back (*it);
	//	}
	m_relation = h.m_relation;
	m_index = h.m_index;
	m_coverage = h.m_coverage;
	m_bChosen = h.m_bChosen;
	m_priority = h.m_priority;
	//m_homo =h.m_homo;
	return *this;
}


void haplotype::printSeq(ostream &os, bool bCut) const
{
	if(bCut||(SNPmap ==0))	{
		for(int i=0; i<gLength; i++)	
			if(getPosition(i))	os << 1;
			else os <<0;		
	}
	else	{
		for(int i=0; i<gOriginLen; i++)	
			if(getPosition(SNPmap[i]))	os << 1;
			else os <<0;	
	}
	return;
}

//compare prioroty of 2 haplotypes, return -1:<; 0:=; 1:>
//order: homo, 1-hete, later-resolved
//in the same type, haplotype with larger m_coverage first; 
//with the same m_coverage, haplotype with larger priority first.
int haplotype::priority(const haplotype *h) const
{
	if(h==xHap)	return 1;
/*	if(m_priority >=9999)	{
		if(m_priority > h->m_priority)		return 1;
		else if(m_priority < h->m_priority)	return -1;
		else if(m_coverage > h->m_coverage) return 1;
		else if(m_coverage < h->m_coverage)	return -1;
		else	return 0;
	}
	if(h->m_priority >=9999)	 return -1;
	if(m_coverage > h->m_coverage) return 1;
	else if(m_coverage < h->m_coverage)	return -1;
	else if(m_priority > h->m_priority)	return 1;
	else if(m_priority < h->m_priority)	return -1;
	else return 0;
*/
	if(m_priority > h->m_priority) return 1;
	else if(m_priority < h->m_priority) return -1;
	else return 0;
}
