/********************************************************
Haplotype Inference by Parsimony
Oct 2002, By Xu Ying
in Dept. of Computer Science, CityU of Hong Kong

input:	genotype file (m sequence * n SNP on {0,1,2})
output:	haplotype (x*n on {0,1,2}, correspodence of m with x)
method: maximum parsimony (minimum number of haplotype - x)
*********************************************************/
#include <fstream>
#include "HaploSet.h"
#include "GenoSet.h"
using namespace std;

int gLength, gSize, gOriginLen, * SNPmap=0;	//gOriginLen make sense only if SNPmap!=0
HaploSet hapset, preHapset; 
GenoSet originGset, gset;
haplotype * xHap=0;
ofstream outf, resultf;
vector<haplotype *> solution;

int main(int argc, char ** argv)	{
	if(argc<=1)		return 0;
	ifstream genofile;
	genofile.open(argv[1]);
	if(!genofile)	{
		cout << "Cannot open file " << argv[1] << endl;
		return 1;
	}			
	outf.open("debug.out");
	resultf.open("myhap.out");
	originGset.readFile(genofile);
	outf << "Input: ";
	originGset.print(outf);
	originGset.cutSNP();
	outf << "after cut identical SNP:" << endl;
	originGset.print(outf);
	//originGset.print(outf, false);
	genofile.close();

	originGset.preResolve();
	if(gset.m_gvec.size()==0)	{
		resultf << preHapset.m_hset.size() << " " << gOriginLen << endl;
		hapit hit;
		for(hit=preHapset.m_hset.begin();hit!=preHapset.m_hset.end(); hit++)	{
			(*hit).printSeq(resultf, false);
			resultf << endl;
		}
		originGset.delGenotypes();
		if(SNPmap!=0)	{
			delete SNPmap;
			SNPmap =0;
		}
		outf.close();
		resultf.close();
		return 1;
	}
	gset.setPreRelation();
	//outf << "after pre-resolve: " << endl;
	//hapset.print(outf);
	//gset.print(outf);
	gset.buildGHgraph();
	//gset.greedyResolve();
	gset.searchResolve();	

	delHset(solution);
	originGset.delGenotypes();
	if(SNPmap!=0)	{
		delete SNPmap;
		SNPmap =0;
	}
	outf.close();
	resultf.close();
	return 0;
}


